"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from honeyhive.types import BaseModel
from typing import Optional, TypedDict
from typing_extensions import NotRequired


class ProjectTypedDict(TypedDict):
    name: str
    description: str
    id: NotRequired[str]


class Project(BaseModel):
    name: str

    description: str

    id: Optional[str] = None
