"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from honeyhive.types import BaseModel
from typing import Optional, TypedDict
from typing_extensions import NotRequired


class UpdateProjectRequestTypedDict(TypedDict):
    project_id: str
    name: NotRequired[str]
    description: NotRequired[str]


class UpdateProjectRequest(BaseModel):
    project_id: str

    name: Optional[str] = None

    description: Optional[str] = None
