"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from honeyhive.models.components import (
    createrunresponse as components_createrunresponse,
)
from honeyhive.types import BaseModel
import httpx
from typing import Optional, TypedDict
from typing_extensions import NotRequired


class CreateRunResponseTypedDict(TypedDict):
    content_type: str
    r"""HTTP response content type for this operation"""
    status_code: int
    r"""HTTP response status code for this operation"""
    raw_response: httpx.Response
    r"""Raw HTTP response; suitable for custom response parsing"""
    create_run_response: NotRequired[
        components_createrunresponse.CreateRunResponseTypedDict
    ]
    r"""Successful response"""


class CreateRunResponse(BaseModel):
    content_type: str
    r"""HTTP response content type for this operation"""

    status_code: int
    r"""HTTP response status code for this operation"""

    raw_response: httpx.Response
    r"""Raw HTTP response; suitable for custom response parsing"""

    create_run_response: Optional[components_createrunresponse.CreateRunResponse] = None
    r"""Successful response"""
